import { CommandHandler } from '@nestjs/cqrs';
import { AuthRepository } from '../../adapters/secondaries/auth.repository';
import { CreateAuthCommand } from '../../commands/create-auth.command';
import { Auth } from '../entities/auth';
import * as bcrypt from 'bcrypt';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';

@CommandHandler(CreateAuthCommand)
export class CreateAuthUseCase {
  constructor(
    private readonly _authRepository: AuthRepository,
    private readonly _usernameRepository: UsernameRepository,
    private readonly _loggingMessager: LoggingMessager,
  ) {}

  async execute(command: CreateAuthCommand): Promise<Auth> {
    const { uuid, password, ...username } = command.createAuthRequest;
    const hash = await bcrypt.hash(password, 10);

    try {
      const auth = await this._authRepository.create({
        uuid,
        password: hash,
      });

      await this._usernameRepository.create({
        uuid,
        ...username,
      });

      return auth;
    } catch (error) {
      this._loggingMessager.publish(
        'auth.create.critical',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
